#include-once
; Gibt die Wochennummer nach ISO 8601 zurueck
; Author: Oscar (www.autoit.de)
Func _GetWeekNumber($iYear = @YEAR, $iMonth = @MON, $iDay = @MDAY)
	Local $bLeapYear, $iDOY, $iWD, $iWNR
	$iDOY = _GetDayOfYear($iYear, $iMonth, $iDay) ; Anzahl der Tage im Jahr ermitteln
	$iWD = _GetWeekday($iYear, $iMonth, $iDay) ; Wochentag ermitteln
	$iWNR = Int(($iDOY - $iWD + 7) / 7) ; die Wochennummer berechnen
	Switch $iWNR
		Case 0 ; wenn die Wochennummer Null ergibt, dann liegt der Tag am Anfang des Jahres (1. Sonderfall)
			$iWD = _GetWeekday($iYear - 1, 12, 31) ; den letzten Wochentag aus dem Vorjahr ermitteln
			$bLeapYear = _IsLeapyear($iYear - 1) ; ermitteln, ob es sich beim Vorjahr um ein Schaltjahr handelt
		Case 52 ; wenn die Wochennummer 52 ergibt, dann liegt der Tag am Ende des Jahres (2. Sonderfall)
			$iWD = _GetWeekday($iYear, 12, 31) ; den letzten Wochentag aus diesem Jahr ermitteln
			$bLeapYear = _IsLeapyear($iYear) ; ermitteln, ob es sich bei diesem Jahr um ein Schaltjahr handelt
		Case Else ; in den anderen Faellen kann die Funktion
			Return $iWNR ; hier verlassen und die Wochennummer zurueckgegeben werden
	EndSwitch
	If $iWD < 4 Then ; wenn der 31.12. vor dem Donnerstag liegt, dann...
		$iWNR = 1 ; ist das die erste Woche des Jahres
	Else ; anderenfalls muss ermittelt werden, ob es eine 53. Woche gibt (3. Sonderfall)
		; wenn der letzte Wochentag auf einen Donnerstag oder,
		; in einem Schaltjahr, auf einen Donnerstag oder Freitag fällt,
		; dann ist das die 53. Woche, ansonsten die 52. Woche.
		$iWNR = (($iWD == 4) Or ($bLeapYear And $iWD == 5)) ? 53 : 52
	EndIf
	Return $iWNR
EndFunc   ;==>_GetWeekNumber

; Gibt den Wochentag nach ISO 8601 (1 = Mo, 2 = Di, 3 = Mi, 4 = Do, 5 = Fr, 6 = Sa, 7 = So) zurueck
; Author: Oscar (www.autoit.de)
Func _GetWeekday($iYear = @YEAR, $iMonth = @MON, $iDay = @MDAY)
	Local Const $aT[] = [0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4]
	$iYear -= $iMonth < 3
	Local $iWD = Mod($iYear + Int($iYear / 4) - Int($iYear / 100) + Int($iYear / 400) + $aT[$iMonth - 1] + $iDay, 7)
	Return ($iWD == 0 ? 7 : $iWD)
EndFunc   ;==>_GetWeekday

; Gibt die Anzahl der Tage (Tag des Jahres) zurueck
; Author: Oscar (www.autoit.de)
Func _GetDayOfYear($iYear = @YEAR, $iMonth = @MON, $iDay = @MDAY)
	Local Const $aMdays[] = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334]
	Return $iDay + $aMdays[$iMonth - 1] + ($iMonth >= 2 And _IsLeapyear($iYear))
EndFunc   ;==>_GetDayOfYear

; Schaltjahrberechnung (true = Schaltjahr, false = kein Schaltjahr)
; Author: Oscar (www.autoit.de)
Func _IsLeapyear($iYear = @YEAR)
	Return Not Mod($iYear, 4) And (Mod($iYear, 100) Or Not Mod($iYear, 400))
EndFunc   ;==>_IsLeapyear
