﻿#include-once
#include <AutoItConstants.au3>
#include <Date.au3>
#include <DateTimeConstants.au3>
#include <GDIPlus.au3>
#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>
#include <WinAPI.au3>
#include <WinAPIGdi.au3>

Global Const $iGuiCtrlCalendar_AppDPI = RegRead('HKEY_CURRENT_USER\Control Panel\Desktop\WindowMetrics', 'AppliedDPI')

;===============================================================================
; Function Name:   _GuiCtrlCalendar_Create
; Description:     Mit dieser Funktion wird ein Kalender auf der GUI erstellt
;                  Der Kalender hat folgende Abmessungen: Breite x Hoehe = 460 x 380 px,
;                  wobei die Hoehe auch 50 px kleiner sein kann (wenn nur 5 Wochen
;                  angezeigt werden).
; Parameter(s):    $iLeft, $iTop = die Gui-Position
;                  $iMonth, $iYear = der anzuzeigende Monat/Jahr
; Requirement(s):  ---
; Return Value(s): Ein Array mit den IDs der Pic-Control-Elemente.
;                  Dieses Array wird fuer die anderen Funktionen benoetigt.
;                  Das Array-Element $aCal[1][7] enthaelt ein Array im Array
;                  Dort werden die Ctrl-IDs der Tage des aktuellen Monats gespeichert,
;                  damit man einzelne Tage im Monat speziell markieren kann.
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _GuiCtrlCalendar_Create($iLeft, $iTop, $iMonth = @MON, $iYear = @YEAR)
	Local $aCal[8][8], $iSizeX = 60, $iSizeY = 50, $iWeekX = 40, $iHeaderY = 30
	Local $iL = $iLeft, $iT = $iTop
	$aCal[0][7] = GUICtrlCreatePic('', $iL, $iT, $iWeekX + $iSizeX * 7, $iSizeY)
	GUICtrlSetState(-1, $GUI_DISABLE)
	$iT += $iSizeY
	For $y = 0 To 6
		$iL = $iLeft
		For $x = 0 To 7
			If $y = 0 Then ; wenn Header-Zeile
				If $x = 0 Then ; wenn Wochennummer-Spalte
					$aCal[$x][$y] = GUICtrlCreatePic('', $iL, $iT, $iWeekX, $iHeaderY)
					$iL += $iWeekX
				Else ; ansonsten (die anderen Spalten)
					$aCal[$x][$y] = GUICtrlCreatePic('', $iL, $iT, $iSizeX, $iHeaderY)
					$iL += $iSizeX
				EndIf
			Else ; ansonsten (die anderen Zeilen)
				If $x = 0 Then ; wenn Wochennummer-Spalte
					$aCal[$x][$y] = GUICtrlCreatePic('', $iL, $iT, $iWeekX, $iSizeY)
					$iL += $iWeekX
				Else ; ansonsten (die anderen Spalten)
					$aCal[$x][$y] = GUICtrlCreatePic('', $iL, $iT, $iSizeX, $iSizeY)
					$iL += $iSizeX
				EndIf
			EndIf
		Next
		$iT += ($y = 0 ? $iHeaderY : $iSizeY)
	Next
	_GuiCtrlCalendar_SetData($aCal, $iMonth, $iYear)
	Return $aCal
EndFunc   ;==>_GuiCtrlCalendar_Create

;===============================================================================
; Function Name:   _GuiCtrlCalendar_Delete
; Description:     Mit dieser Funktion kann ein Kalender von der GUI entfernt werden
; Parameter(s):    $aCal ist das Rueckgabe-Array von _GuiCtrlCalendar_Create
; Requirement(s):  ---
; Return Value(s): Bei Erfolg = 1, ansonsten 0 und @error = 1
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _GuiCtrlCalendar_Delete(ByRef $aCal)
	If Not IsArray($aCal) Or UBound($aCal) <> 8 Then Return SetError(1, 0, 0)
	GUICtrlDelete($aCal[0][7])
	For $y = 0 To 6
		For $x = 0 To 7
			GUICtrlDelete($aCal[$x][$y])
		Next
	Next
	$aCal = 0
	Return 1
EndFunc   ;==>_GuiCtrlCalendar_Delete

;===============================================================================
; Function Name:   _GuiCtrlCalendar_DrawText
; Description:     Mit dieser Funktion wird der Text zentriert in die Pic-Ctrls
;                  eingetragen (funktioniert mit jedem Pic-Control).
; Parameter(s):    $idPic = die Ctrl-ID (Rueckgabe von GuiCtrlCreatePic)
;                  $sText = ein beliebiger Text
;                  $iFontSize = die Zeichensatzgroesse (siehe _GDIPlus_FontCreate)
;                  $iFontStyle = der Zeichensatzstil (siehe _GDIPlus_FontCreate)
;                  $iTextColor = Vordergrundfarbe (RGB) fuer den Text
;                  $iBkColor = Hintergrundfarbe (RGB) fuer den Text
;                  $sToolTip = der Text der beim Tooltip angezeigt werden soll
; Requirement(s):  #include <GDIPlus.au3>
;                  #include <WinAPI.au3>
; Return Value(s): Bei Erfolg = 1, ansonsten 0 und @error = 1
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _GuiCtrlCalendar_DrawText($idPic, $sText, $iFontSize, $iFontStyle = 0, $iTextColor = 0x000000, $iBkColor = 0xFFFFFF, $sToolTip = '')
	Local $hPic = GUICtrlGetHandle($idPic) ; das Handle vom Pic-Control holen
	If Not IsHWnd($hPic) Then Return SetError(1, 0, 0) ; wenn kein Handle, dann Funktion mit Fehlermeldung verlassen
	$iFontSize = $iFontSize / $iGuiCtrlCalendar_AppDPI * 96
	GUICtrlSetTip($idPic, $sToolTip, '', $TIP_NOICON, $TIP_BALLOON)
	_GDIPlus_Startup()
	Local $hParent = _WinAPI_GetParent($hPic) ; das Fenster des Pic-Controls ermitteln
	Local $aPos = ControlGetPos($hParent, '', $idPic) ; die Koordinaten und Abmessungen des Pic-Control ermitteln
	Local $hBitmap = _GDIPlus_BitmapCreateFromScan0($aPos[2], $aPos[3]) ; Eine Bitmap erstellen
	Local $hGfxCtxt = _GDIPlus_ImageGetGraphicsContext($hBitmap) ; Graphic-Context der Bitmap holen
	_GDIPlus_GraphicsSetInterpolationMode($hGfxCtxt, $GDIP_INTERPOLATIONMODE_HIGHQUALITYBICUBIC)
	_GDIPlus_GraphicsClear($hGfxCtxt, BitOR(0xFF000000, $iBkColor)) ; Die Bitmap mit der Hintergrundfarbe füllen
	Local $hBrush = _GDIPlus_BrushCreateSolid(BitOR(0xFF000000, $iTextColor))
	Local $hFormat = _GDIPlus_StringFormatCreate()
	_GDIPlus_StringFormatSetAlign($hFormat, 1) ; 1 = horizontal zentriert
	_GDIPlus_StringFormatSetLineAlign($hFormat, 1) ; 1 = vertikal entriert
	Local $hFamily = _GDIPlus_FontFamilyCreate('Arial')
	Local $hFont = _GDIPlus_FontCreate($hFamily, $iFontSize, $iFontStyle)
	Local $tLayout = _GDIPlus_RectFCreate(0, 0, $aPos[2], $aPos[3])
	Local $aInfo = _GDIPlus_GraphicsMeasureString($hGfxCtxt, $sText, $hFont, $tLayout, $hFormat)
	_GDIPlus_GraphicsDrawStringEx($hGfxCtxt, $sText, $hFont, $aInfo[0], $hFormat, $hBrush)
	Local $hBMP = _GDIPlus_BitmapCreateDIBFromBitmap($hBitmap) ; HBitmap von der Bitmap erstellen
	_WinAPI_DeleteObject(GUICtrlSendMsg($idPic, $STM_SETIMAGE, $IMAGE_BITMAP, $hBMP)) ; und an das Pic-Control senden
	_WinAPI_DeleteObject($hBMP)
	_GDIPlus_FontDispose($hFont)
	_GDIPlus_FontFamilyDispose($hFamily)
	_GDIPlus_StringFormatDispose($hFormat)
	_GDIPlus_BrushDispose($hBrush)
	_GDIPlus_GraphicsDispose($hGfxCtxt)
	_GDIPlus_BitmapDispose($hBitmap)
	_GDIPlus_Shutdown()
	Return 1
EndFunc   ;==>_GuiCtrlCalendar_DrawText

;===============================================================================
; Function Name:   _GuiCtrlCalendar_GetDays
; Description:     Mit dieser Funktion wird das Array im Array extrahiert
;                  Das enthaelt die Ctrl-IDs der Tage im aktuellen Monat und
;                  wird fuer _GuiCtrlCalendar_DrawText benoetigt.
; Parameter(s):    $aCal ist das Rueckgabe-Array von _GuiCtrlCalendar_Create
; Requirement(s):  ---
; Return Value(s): Bei Erfolg = 1, ansonsten 0 und @error = 1
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _GuiCtrlCalendar_GetDays(ByRef $aCal)
	If Not IsArray($aCal) Or UBound($aCal) <> 8 Then Return SetError(1, 0, 0)
	Local $aDays = $aCal[1][7] ; <- Achtung! Array im Array extrahieren
	Return $aDays ; und zurueckgeben
EndFunc   ;==>_GuiCtrlCalendar_GetDays

;===============================================================================
; Function Name:   _GuiCtrlCalendar_SetData
; Description:     Mit dieser Funktion wird der Monat fuer den Kalender geaendert
; Parameter(s):    $aCal ist das Rueckgabe-Array von _GuiCtrlCalendar_Create
; Requirement(s):  #include <WinAPI.au3>
;                  #include <WinAPIGdi.au3>
;                  #include <Date.au3>
; Return Value(s): Bei Erfolg = 1, ansonsten 0 und @error = 1
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _GuiCtrlCalendar_SetData(ByRef $aCal, $iMonth = @MON, $iYear = @YEAR)
	If Not IsArray($aCal) Or UBound($aCal) <> 8 Then Return SetError(1, 0, 0)
	Local $hDate = GUICtrlGetHandle($aCal[0][7]) ; das Handle von der Ueberschrift holen
	Local $hParent = _WinAPI_GetParent($hDate) ; das Fenster des Pic-Controls ermitteln
	_WinAPI_LockWindowUpdate($hParent)
	Local $iLeapyear = _DateIsLeapYear($iYear)
	Local Const $aHeaderText[8] = ['KW', 'Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa', 'So']
	;           $aHeaderColor[] = [   KW,       Mo,       Di,       Mi,       Do,       Fr,       Sa,       So   ]
	Local Const $aHeaderColor[] = [0x444444, 0x4444AA, 0x4444AA, 0x4444AA, 0x4444AA, 0x4444AA, 0xEEAA44, 0x22AA22]
	;     $aDaysPerMonth[] = [Dez,Jan,Feb+ $iLeapyear,Mar,Apr,Mai,Jun,Jul,Aug,Sep,Okt,Nov,Dez]
	Local $aDaysPerMonth[] = [31, 31, 28 + $iLeapyear, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
	Local $iDaysNow = $aDaysPerMonth[$iMonth], $iDaysPrev = $aDaysPerMonth[$iMonth - 1]
	Local $iStartWeekday = _DateToDayOfWeekISO($iYear, $iMonth, 1), $iDay = ($iStartWeekday = 1 ? 1 : $iDaysPrev - $iStartWeekday + 2)
	Local $fMonthNow = ($iStartWeekday = 1), $fSixWeek = ($iStartWeekday = 6 And $iDaysNow = 31) Or ($iStartWeekday = 7 And $iDaysNow >= 30)
	Local $aMonthDays[$iDaysNow + 1] = [$fSixWeek]
	; $aCal[0][7] wird fuer die Ctrl-ID der Ueberschrift verwendet
	_GuiCtrlCalendar_DrawText($aCal[0][7], StringFormat('%s %04d', _DateToMonth($iMonth, $DMW_LOCALE_LONGNAME), $iYear), 28, 1, 0x000000, 0xFFFFFF)
	For $x = 0 To 7
		_GuiCtrlCalendar_DrawText($aCal[$x][0], $aHeaderText[$x], ($x = 0 ? 10 : 18), 1, 0xFFFFFF, $aHeaderColor[$x])
		GUICtrlSetState($aCal[$x][6], $fSixWeek ? $GUI_SHOW : $GUI_HIDE)
	Next
	For $y = 1 To 6
		For $x = 0 To 7
			If $x = 0 Then ; wenn Spalte fuer die Kalenderwoche
				If $y = 1 And $iStartWeekday > 1 Then ; wenn der Monat nicht mit einem Montag beginnt, dann...
					If $iMonth = 1 Then ; wenn Januar, dann...
						_GuiCtrlCalendar_DrawText($aCal[$x][$y], _WeekNumberISO($iYear - 1, 12, $iDaysPrev), 12, 0, 0x444444, 0xDDDDDD)
					Else ; ansonsten (Februar bis Dezember)
						_GuiCtrlCalendar_DrawText($aCal[$x][$y], _WeekNumberISO($iYear, $iMonth - 1, $iDaysPrev), 12, 0, 0x444444, 0xDDDDDD)
					EndIf
				Else ; ansonsten (Monat beginnt mit einem Montag)
					_GuiCtrlCalendar_DrawText($aCal[$x][$y], _WeekNumberISO($iYear, $iMonth, $iDay), 12, 0, 0x444444, 0xDDDDDD)
				EndIf
			Else ; ansonsten (die Spalten fuer die Tage)
				_GuiCtrlCalendar_DrawText($aCal[$x][$y], $iDay, $fMonthNow ? 24 : 14, 0, $fMonthNow ? $aHeaderColor[$x] : 0xAAAAAA)
				If $fMonthNow And $iDay >= 1 And $iDay <= $iDaysNow Then $aMonthDays[$iDay] = $aCal[$x][$y]
				$iDay += 1
				If $y = 1 And $iDay > $iDaysPrev Then
					$iDay = 1
					$fMonthNow = True
				EndIf
				If $y > 1 And $iDay > $iDaysNow Then
					$iDay = 1
					$fMonthNow = False
					$iMonth += 1
					If $iMonth > 12 Then
						$iMonth = 1
						$iYear += 1
					EndIf
				EndIf
			EndIf
		Next
	Next
	$aCal[1][7] = $aMonthDays ; <- Achtung! Speichere Array im Array
	_WinAPI_LockWindowUpdate(0)
	Return 1
EndFunc   ;==>_GuiCtrlCalendar_SetData

;===============================================================================
; Function Name:   _GuiCtrlCalendar_SetState
; Description:     Mit dieser Funktion kann ein Kalender angezeigt/versteckt werden
; Parameter(s):    $aCal ist das Rueckgabe-Array von _GuiCtrlCalendar_Create
; Requirement(s):  ---
; Return Value(s): Bei Erfolg = 1, ansonsten 0 und @error = 1
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _GuiCtrlCalendar_SetState(ByRef $aCal, $fShow)
	If Not IsArray($aCal) Or UBound($aCal) <> 8 Then Return SetError(1, 0, 0)
	Local $aDays = $aCal[1][7], $fSixWeek = $aDays[0]
	GUICtrlSetState($aCal[0][7], $fShow ? $GUI_SHOW : $GUI_HIDE)
	For $y = 0 To 6
		For $x = 0 To 7
			If $y = 6 Then
				GUICtrlSetState($aCal[$x][$y], $fSixWeek And $fShow ? $GUI_SHOW : $GUI_HIDE)
			Else
				GUICtrlSetState($aCal[$x][$y], $fShow ? $GUI_SHOW : $GUI_HIDE)
			EndIf
		Next
	Next
	Return 1
EndFunc   ;==>_GuiCtrlCalendar_SetState
